/*  libbadpenguin -- generic library for Bad Penguin development
 *
 *  Copyright (C) 1996 Antonio Gallo
 *  agx@linux.it
 *  
 *  Newer versions should be on www.badpenguin.org
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 */

#include <config.h>

#include <sys/stat.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <libintl.h>
#define _(a)	dgettext(PACKAGE, a)

#include "badpenguin-ansi.h"
#include "badpenguin-errors.h"
#include "badpenguin-fs.h"

int move_dev( char* source, char* target) {
  struct stat lpstat;
  
  // Try to rename
  if ( 0 == rename(source,target) ) return 0;
  
  // Move
  lstat( source, &lpstat );
  if ( 0 != mknod(target, lpstat.st_mode, lpstat.st_rdev) ) {
    printERROR(_("move_dev: unable to create device file %s\n"), target);
    normal_color();      
  }  
  // apply
  if ( 0 != file_time( target, lpstat.st_mtime, lpstat.st_atime ) ) {
    printERROR(_("move_dev: unable to set file access time for %s\n"), target);
    normal_color();
    unlink(target);
  }  
  if ( 0 != chmod( target, lpstat.st_mode & 0xFFF ) ) {
    printERROR(_("move_dev: unable to set file permissions for %s\n"), target);
    normal_color();
    unlink(target);
  }
  if ( 0 != chown( target, lpstat.st_uid, lpstat.st_gid )) {
    printERROR(_("move_dev: unable to set UID and/or GID for %s\n"), target);
    normal_color();        
    unlink(target);
  }
  if ( 0 != unlink(source) ) {
    printERROR(_("move_dev: unable to remove old device file for %s\n"), target);
    normal_color();        
    return -1;
  }
  return 0;
}

